/*
 * $Id: BasicDesktopFrame.java 91606 2009-05-18 14:18:43Z clinio $
 */

package csgrid.client.desktop;

import csbase.client.Client;
import csbase.client.applicationmanager.ApplicationManager;
import csbase.client.applicationmanager.ApplicationManagerMenuUtilities;
import csbase.client.applicationmanager.ApplicationPanel;
import csbase.client.applicationmanager.ClearApplicationCacheAction;
import csbase.client.applications.ApplicationImages;
import csbase.client.desktop.AboutAction;
import csbase.client.desktop.DesktopFrame;
import csbase.client.desktop.DesktopPref;
import csbase.client.desktop.ProjectInfoAction;
import csbase.client.desktop.RemoteTask;
import csbase.client.desktop.TreeFilterPanelListener;
import csbase.client.ias.AdminFrame;
import csbase.client.ias.AdminModifyUserInfoDialog;
import csbase.client.ias.ModifyUserInfoDialog;
import csbase.client.ias.UserInfoDialog;
import csbase.client.kernel.ClientException;
import csbase.client.preferences.PreferenceCategory;
import csbase.client.project.BasicProjectTreeFactory;
import csbase.client.project.ProjectLocationBar;
import csbase.client.project.ProjectTreeAdapter;
import csbase.client.project.ProjectTreeFactory;
import csbase.client.project.action.CommonProjectCreationAction;
import csbase.client.project.action.CommonProjectOpenAction;
import csbase.client.project.action.CommonProjectUpdateAction;
import csbase.client.project.action.ProjectCloseAction;
import csbase.client.project.action.ProjectDeleteAction;
import csbase.client.project.action.ProjectHistoryShowAction;
import csbase.client.project.action.ProjectTreeUpdateAction;
import csbase.client.project.action.ProjectUsersManagementAction;
import csbase.client.project.action.ShowHiddenFilesAction;
import csbase.client.project.action.ShowTreeFilterAction;
import csbase.client.remote.ClientRemoteMonitor;
import csbase.client.remote.ProjectAdminObserver;
import csbase.client.remote.srvproxies.ApplicationManagementProxy;
import csbase.logic.AlgorithmAdminPermission;
import csbase.logic.CommonClientProject;
import csbase.logic.CommonProjectInfo;
import csbase.logic.ProjectUserEvent;
import csbase.logic.SharedProjectEvent;
import csbase.logic.User;
import csbase.remote.ClientRemoteLocator;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import tecgraf.javautils.core.lng.LNG;
import tecgraf.javautils.gui.GUIUtils;
import tecgraf.javautils.gui.SwingThreadDispatcher;

/**
 * A classe <code>CSGridDesktopFrame</code> implementa a interface principal do sistema no lado
 * cliente. Ela provê a funcionalidade mínima comum a qualquer sistema desenvolvido com base no
 * CSBase. A configuração deste componente (específica para cada sistema) compreende: - o título do
 * desktop e a imagem a ele associada - a localização dos arquivos de propriedades que configuram as
 * aplicações e os grupos de aplicações.
 *
 * @author Tecgraf/PUC-Rio
 */
public class CSGridDesktopFrame extends DesktopFrame implements Observer {
  /** Botão para selecionar projeto corrente (toolbar) */
  private JButton openProjButton;

  /** Item de menu para abertura de projeto de um usuário */
  private JMenuItem openUserProjMenuItem;

  /** Item de Menu para Remoção de projeto */
  private JMenuItem removeProjMenuItem;

  /** Painel de aplicações. */
  private ApplicationPanel applicationPanel;
  /**
   * Texto que será adicionado ao final do título das janelas dependentes do tipo
   * DesktopComponentFrame.
   */
  private String appendedTitle;

  /** Id da aplicação Gerenciador de Algoritmos */
  private static final String ALGORITHMS_MANAGER_APPLICATION_ID = "algorithmsmanager";

  /**
   * Constrói a interface principal do cliente.
   *
   * @throws ClientException
   */
  public CSGridDesktopFrame() throws ClientException {
    setTitle(Client.getInstance().getSystemName());
    /* Instala a árvore do projetos no desktop */
    installProjectTree();
    /* Constrói os painéis do desktop, a barra de ferramentas e o menu */
    JFrame mainFrame = getDesktopFrame();
    JComponent mainPanel =
        composeMainPanel(
            createProjectPanel(),
            createDirectoryContentsPanel(),
            createTreeFilterPanel(),
            createApplicationPanel(),
            createNotificationPanel());
    JPanel firstPanel = new JPanel(new BorderLayout());
    firstPanel.add(createToolBar(), BorderLayout.NORTH);
    firstPanel.add(mainPanel, BorderLayout.CENTER);
    mainFrame.getContentPane().add(firstPanel);
    /* Cria um menu principal do desktop */
    createDesktopMenuBar();
    /* Cadastra o desktop como observador de projetos */
    ProjectAdminObserver.getInstance().addObserver(this);
    /* Salva as configurações do projeto */
    mainFrame.addWindowListener(
        new WindowAdapter() {
          @Override
          public void windowClosing(WindowEvent e) {
            exitApplication();
          }
        });
  }

  /**
   * Método para ajustar o título da janela para ter a versão do CSGrid.
   *
   * @param title título
   */
  @Override
  public String adjustTitle(String title) {
    if (appendedTitle == null) {
      final String version = Client.getInstance().getVersion();
      final User user = User.getLoggedUser();
      final String systemName = LNG.get("SERVER");

      String serverName = null;
      final RemoteTask<String> task =
          new RemoteTask<String>() {
            @Override
            public void performTask() throws Exception {
              setResult(ClientRemoteLocator.server.getSystemName());
            }
          };
      // Esse diálogo não deve aparecer rapidamente pois dará uma impressão
      // ruim ao usuário no start-up do desktop. Ele somente aparecerá caso
      // haja alguma demora real na conexão com o servidor.
      task.setProgressDialogDelay(10);
      task.execute(this.getDesktopFrame(), "", systemName + "...");
      if (!task.getStatus()) {
        serverName = "ERROR";
      }
      serverName = task.getResult();
      if (serverName == null) {
        serverName = "[" + systemName + "]";
      }
      final String userLogin = user.getLogin();
      appendedTitle = userLogin + " :: " + serverName + " (" + version + ")";
    }
    return title + " - " + appendedTitle;
  }

  /**
   * Cria o painel que contém a árvore de projetos.
   *
   * @return o painel com a árvore de projetos
   * @throws ClientException caso ocorra algum problema na criação da árvore
   */
  private JComponent createProjectPanel() throws ClientException {
    JComponent scrollPane = new JScrollPane(getTree().getTree());
    projectTreePanel = new JPanel(new BorderLayout());
    projectTreePanel.add(scrollPane, BorderLayout.CENTER);
    ProjectLocationBar locationBar = new ProjectLocationBar(projectTree);
    projectTreePanel.add(locationBar, BorderLayout.NORTH);
    return projectTreePanel;
  }

  /**
   * Cria o painel que contém as janelas com as aplicações, agrupadas por categoria.
   *
   * @return o painel com as aplicações
   * @throws ClientException caso ocorra algum problema na criação da árvore
   */
  private JComponent createApplicationPanel() throws ClientException {
    applicationPanel = new ApplicationPanel();
    applicationPanel.addProjectTreeListernerForLayoutAutoPersistency(getTree());
    applicationPanel.addMouseListenerForLayoutMenu();
    applicationPanel.addMouseListenerForApplicationsMenu();
    return applicationPanel;
  }

  /**
   * Compõe o painel principal, separando os diferentes subpainéis com SplitPanes.
   *
   * @param projectTreePanel painel com a árvore de projeto
   * @param dirContentsPanel painel com detalhes do diretório selecionado
   * @param filterPanel painel de filtro da árvore de projeto
   * @param appPanel painel de aplicações
   * @param notificationPanel painel de notificações
   * @return o painel principal do desktop
   */
  private JComponent composeMainPanel(
      JComponent projectTreePanel,
      JComponent dirContentsPanel,
      JComponent filterPanel,
      JComponent appPanel,
      JComponent notificationPanel) {
    /*
     * Compõe um painel com a árvore de diretórios e os detalhes do diretório
     * corrente, com "arrumação" vertical
     */
    splitPaneBetweenTreeAndTable =
        new JSplitPane(JSplitPane.VERTICAL_SPLIT, projectTreePanel, dirContentsPanel);
    splitPaneBetweenTreeAndTable.setOneTouchExpandable(true);
    splitPaneBetweenTreeAndTable.setResizeWeight(0.75);
    JPanel panel = new JPanel(new BorderLayout());
    panel.add(treeFilterPanel, BorderLayout.NORTH);
    panel.add(splitPaneBetweenTreeAndTable, BorderLayout.CENTER);

    /* Compõe este painel com o de aplicações, com "arrumação" horizontal */
    splitPaneBetweenTreeAndApp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, panel, appPanel);
    splitPaneBetweenTreeAndApp.setOneTouchExpandable(true);
    splitPaneBetweenTreeAndApp.setResizeWeight(0.0);

    /* Acrescenta o painel de notificações, com "arrumação" vertical */
    splitPaneBetweenAppAndNotif =
        new JSplitPane(JSplitPane.VERTICAL_SPLIT, splitPaneBetweenTreeAndApp, notificationPanel);
    splitPaneBetweenAppAndNotif.setOneTouchExpandable(true);
    splitPaneBetweenAppAndNotif.setResizeWeight(1.0);
    getDesktopFrame().setResizable(true);
    return splitPaneBetweenAppAndNotif;
  }

  /**
   * Cria a barra de ferramentas do desktop. Esse elemento é composto por botões para a seleção
   * (open) de um projeto, e para a configuração das janelas que contém grupos de aplicações.
   * (opções: em cascata, vertical ou horizontal)
   *
   * @return a barra de ferramentas
   */
  private JToolBar createToolBar() {
    JToolBar toolBar = new JToolBar();
    /* Abrir projeto */
    openProjButton = new JButton(new CommonProjectOpenAction(projectTree));
    openProjButton.setIcon(ApplicationImages.ICON_OPEN_16);
    openProjButton.setToolTipText(LNG.get("desktop.tooltip.open.project"));
    openProjButton.setText("");
    openProjButton.setEnabled(
        User.getLoggedUser().isAdmin() || userHasProject() || userHasOthersProject());
    toolBar.add(openProjButton);
    /* Criar projeto */
    JButton createButton = new JButton(new CommonProjectCreationAction(projectTree));
    createButton.setIcon(ApplicationImages.ICON_NEW_16);
    createButton.setToolTipText(LNG.get("desktop.tooltip.create.project"));
    createButton.setText("");
    createButton.setEnabled(true);
    toolBar.add(createButton);
    /* Habilitar/Desabilitar a exibição de filtro na árvore de projetos */
    final JToggleButton showTreeFilterButton =
        new JToggleButton(new ShowTreeFilterAction(treeFilterPanel));
    showTreeFilterButton.setIcon(ApplicationImages.ICON_FILTER_16);
    showTreeFilterButton.setText("");
    showTreeFilterButton.setToolTipText(LNG.get("desktop.tooltip.showTreeFilterButton"));
    treeFilterPanel.addListener(
        new TreeFilterPanelListener() {
          @Override
          public void visibilityChanged(boolean panelVisible) {
            showTreeFilterButton.setSelected(panelVisible);
          }
        });
    toolBar.add(showTreeFilterButton);
    /* --------- */
    toolBar.addSeparator();
    /* Executar algoritmo */
    JButton execButton = toolBar.add(applicationPanel.getApplicationAction("executor"));
    execButton.setEnabled(false);
    execButton.setToolTipText(LNG.get("desktop.tooltip.executor"));
    enableIfProjectOpened(execButton);
    /* Editor de Fluxos */
    JButton flowButton = toolBar.add(applicationPanel.getApplicationAction("flowapplication"));
    flowButton.setToolTipText(LNG.get("desktop.tooltip.flowapplication"));
    /* Monitorar comandos */
    JButton cmdMonitorButton =
        toolBar.add(applicationPanel.getApplicationAction("commandsmonitor"));
    cmdMonitorButton.setToolTipText(LNG.get("desktop.tooltip.commandsmonitor"));
    /* Gerenciar Algoritmos */
    User user = User.getLoggedUser();
    AlgorithmAdminPermission algoPermission = user.getPermission(AlgorithmAdminPermission.class);
    if (user.isAdmin() || (algoPermission != null)) {
      JButton algoManagerButton =
          toolBar.add(applicationPanel.getApplicationAction(ALGORITHMS_MANAGER_APPLICATION_ID));
      algoManagerButton.setToolTipText(LNG.get("desktop.tooltip.algorithmsmanager"));
    }
    /* --------- */
    toolBar.addSeparator();
    /* Monitorar servidores */
    JButton srvMonitorButton = toolBar.add(applicationPanel.getApplicationAction("sgamonitor"));
    srvMonitorButton.setToolTipText(LNG.get("desktop.tooltip.sgamonitor"));
    /* --------- */
    toolBar.addSeparator();
    /* Cascatear janelas */
    JButton cascadeButton = new JButton();
    cascadeButton.setIcon(ApplicationImages.ICON_WINCASCADE_16);
    cascadeButton.setToolTipText(LNG.get("desktop.tooltip.cascade"));
    cascadeButton.addActionListener(
        new ActionListener() {
          @Override
          public void actionPerformed(ActionEvent e) {
            applicationPanel.cascadeGroups();
          }
        });
    toolBar.add(cascadeButton);

    /* Janelas na vertical */
    JButton tileVertButton = new JButton();
    tileVertButton.setIcon(ApplicationImages.ICON_WINTILEVERTICAL_16);
    tileVertButton.setToolTipText(LNG.get("desktop.tooltip.tile.vertical"));
    tileVertButton.addActionListener(
        new ActionListener() {
          @Override
          public void actionPerformed(ActionEvent e) {
            applicationPanel.tileGroupsVertical();
          }
        });
    toolBar.add(tileVertButton);
    /* Janelas na horizontal */
    JButton tileHorButton = new JButton();
    tileHorButton.setIcon(ApplicationImages.ICON_WINTILEHORIZONTAL_16);
    tileHorButton.setToolTipText(LNG.get("desktop.tooltip.tile.horizontal"));
    tileHorButton.addActionListener(
        new ActionListener() {
          @Override
          public void actionPerformed(ActionEvent e) {
            applicationPanel.tileGroupsHorizontal();
          }
        });
    toolBar.add(tileHorButton);
    /* --------- */
    toolBar.addSeparator();
    /* Sobre o sistema */
    JButton aboutButton = new JButton(new AboutAction());
    toolBar.add(aboutButton);
    return toolBar;
  }

  /**
   * Constrói a barra de menu do desktop.
   *
   * @throws ClientException Caso ocorra falha na criação do menu
   */
  private void createDesktopMenuBar() throws ClientException {
    JMenuBar menuBar = new JMenuBar();
    Map<Integer, JMenu> menus = new HashMap<Integer, JMenu>();
    int order = 1;
    /* Menu de Projeto */
    boolean projectMenuVisible =
        this.getConfiguration().getOptionalBooleanProperty("project.menu.visible", true);
    order = this.getConfiguration().getOptionalIntegerProperty("project.menu.order", order);
    if (projectMenuVisible) {
      menus.put(order++, createProjectMenu());
    }
    /* Menu de Algoritmos */
    boolean algorithmMenuVisible =
        this.getConfiguration().getOptionalBooleanProperty("algorithm.menu.visible", true);
    order = this.getConfiguration().getOptionalIntegerProperty("algorithm.menu.order", order);
    if (algorithmMenuVisible) {
      menus.put(order++, createAlgorithmMenu());
    }
    /* Menu de gerenciamento dos SGAs */
    boolean sgaMenuVisible =
        this.getConfiguration().getOptionalBooleanProperty("sga.menu.visible", true);
    order = this.getConfiguration().getOptionalIntegerProperty("sga.menu.order", order);
    if (sgaMenuVisible) {
      menus.put(order++, createSGAMenu());
    }
    /* Menu de Aplicações */
    boolean applicationMenuVisible =
        this.getConfiguration().getOptionalBooleanProperty("application.menu.visible", true);
    order = this.getConfiguration().getOptionalIntegerProperty("application.menu.order", order);
    if (applicationMenuVisible) {
      menus.put(
          order++, ApplicationManagerMenuUtilities.buildMenu(LNG.get("desktop.applications")));
    }
    /* Menu de Configuração */
    boolean configurationMenuVisible =
        this.getConfiguration().getOptionalBooleanProperty("config.menu.visible", true);
    order = this.getConfiguration().getOptionalIntegerProperty("config.menu.order", order);
    if (configurationMenuVisible) {
      menus.put(order++, createConfigurationMenu());
    }
    /* Menu de Ajuda */
    boolean helpMenuVisible =
        this.getConfiguration().getOptionalBooleanProperty("help.menu.visible", true);
    order = this.getConfiguration().getOptionalIntegerProperty("help.menu.order", order);
    if (helpMenuVisible) {
      menus.put(order++, createHelpMenu());
    }
    /* Monta a barra de menu de acordo com a configuração definida */
    for (JMenu menu : menus.values()) {
      menuBar.add(menu);
    }
    setMenuBar(menuBar);
  }

  /**
   * Constrói o menu para gerenciamente dos projetos do usuário.
   *
   * @return o menu de gerenciamente de projetos.
   */
  protected JMenu createProjectMenu() {
    JMenu menu = new JMenu(LNG.get("desktop.project"));
    boolean hasProject = userHasProject();
    boolean hasOtherProjects = User.getLoggedUser().isAdmin() || userHasOthersProject();
    /* Ícone vazio para alinharmos pelo checkbox do filtro */
    Icon emptyIcon = GUIUtils.createEmptyIcon(6, 6);
    /* Abertura de projeto */
    menu.add(createOpenMenuItem(hasProject || hasOtherProjects));
    /* Criação de projeto */
    menu.add(createProjectCreationItem());
    /* Remoção do projeto */
    menu.add(createProjectRemoveItem(hasProject)).setIcon(emptyIcon);
    /* -------- */
    menu.addSeparator();
    /* Exibição do painel de filtro para a árvore de projetos */
    menu.add(createShowTreeFilterItem());
    /* Controle da exibição de arquivos ocultos. */
    menu.add(createShowHiddenFilesMenuItem());
    /* Atualização automática de projeto */
    menu.add(createProjectAutoUpdateItem());
    /* -------- */
    menu.addSeparator();
    /* Ver informações */
    menu.add(createProjectInfoItem()).setIcon(emptyIcon);
    /* Ver histórico do projeto */
    menu.add(createProjectHistoryItem()).setIcon(emptyIcon);
    /* Atualização de informações do projeto */
    menu.add(createProjectUpdateItem()).setIcon(emptyIcon);
    /* Atualização da árvore do projeto */
    menu.add(createUpdateTreeItem()).setIcon(emptyIcon);
    /* Gerenciamento dos usuários de um projeto */
    menu.add(createUsersManagementMenuItem()).setIcon(emptyIcon);
    /* Fechar projeto */
    menu.add(createProjectCloseMenuItem()).setIcon(emptyIcon);
    /* -------- */
    menu.addSeparator();
    /* Sair do sistema */
    menu.add(createExitMenuItem());
    return menu;
  }

  /**
   * Constrói o menu para algoritmos (execução e monitoração)
   *
   * @return o menu de algoritmos
   */
  protected JMenu createAlgorithmMenu() {
    JMenu menu = new JMenu(LNG.get("desktop.algorithms"));
    /* Execução de algoritmo */
    menu.add(createExecuteMenuItem());
    /* Construtor de fluxos */
    menu.add(createAlgoFlowMenuItem());
    /* Monitoração da execução de algoritmos */
    menu.add(createComandMonitoringMenuItem());
    /* -------- */

    User user = User.getLoggedUser();
    AlgorithmAdminPermission algoPermission = user.getPermission(AlgorithmAdminPermission.class);
    if (user.isAdmin() || (algoPermission != null)) {
      menu.addSeparator();
      /* Aplicação Gerenciador de Algoritmos */
      menu.add(createAlgorithmsAdministrationMenuItem());
    }
    return menu;
  }

  /**
   * Constrói o menu para gerenciamento e monitaração de SGAs (máquinas para execução remota de
   * algoritmos)
   *
   * @return menu de gerenciamento de SGAs
   */
  private JMenu createSGAMenu() {
    JMenu menu = new JMenu(LNG.get("desktop.menu.servers"));
    /* Gerenciamento e monitoração de servidores */
    menu.add(createServersMenuItem());
    return menu;
  }

  /**
   * Constrói o menu para as opções de configuração do desktop e do servidor
   *
   * @return o menu de configuração.
   */
  protected JMenu createConfigurationMenu() {
    JMenu menu = new JMenu(LNG.get("desktop.menu.config"));
    /* Configurar usuário corrente */
    menu.add(createUserConfigItem());
    /* Configurar o painel das aplicações */
    menu.add(applicationPanel.createGroupsConfigurationMenu());
    /* Restaurar o desktop */
    menu.add(createRestoreDesktopItem());

    /* Item exclusivo do administrador */
    if (User.getLoggedUser().isAdmin()) {
      /* -------- */
      menu.addSeparator();
      menu.add(createAdminMenuItem());
      menu.add(createLogAdministrationMenuItem());
      menu.add(new JMenuItem(applicationPanel.getApplicationAction("statsviewer")));

      menu.addSeparator();
      menu.add(createApplicationReloadMenuItem());
    } else {
      menu.addSeparator();
    }
    /* Limpeza de cache de aplicações. */
    menu.add(new ClearApplicationCacheAction());

    /* Editor das preferências. */
    menu.addSeparator();
    menu.add(createEditPrefItem());
    return menu;
  }

  /**
   * Cria item de menu que permite o usuário editar todas as suas preferências.
   *
   * @return item de menu que permite o usuário editar todas as suas preferências.
   */
  private JMenuItem createEditPrefItem() {
    final ApplicationManager appMgr = ApplicationManager.getInstance();

    AbstractAction prefAction = appMgr.createApplicationAction("preferenceviewer");
    return new JMenuItem(prefAction);
  }

  /**
   * Constrói o menu de ajuda.
   *
   * @return o menu de ajuda.
   */
  protected JMenu createHelpMenu() {
    JMenu menu = new JMenu(LNG.get("desktop.menu.help"));
    /* About */
    menu.add(createAboutMenuItem());
    return menu;
  }

  /** Constrói e apresenta os componentes da interface principal do cliente. */
  @Override
  public void showDesktop() {
    super.showDesktop();
    applicationPanel.tileGroupsHorizontal();
  }

  /**
   * Fecha o projeto corrente (caso exista) antes de encerrar a aplicação cliente principal.
   *
   * @param projCloseAction ação de fechamento do projeto
   */
  public void shutdownDesktop(ProjectCloseAction projCloseAction) {
    if (ClientRemoteMonitor.getInstance().isAlive()) {
      // Só fecha o projeto se houver conexão com o servidor.
      projCloseAction.close();
    }
    shutdownDesktop();
  }

  /**
   * Este método é chamado nas seguintes situações: 1 - Um projeto do usuário é criado 2 - Um
   * projeto do usuário é removido 3 - O usuário é adicionado ou removido de um projeto 4 - Um
   * projeto do qual o usuário participa é removido Desta forma, habilita ou desabilita os itens do
   * menu Projeto "(Abrir) Do Usuário" e "(Abrir) De outros usuários".
   *
   * @param o .
   * @param arg .
   */
  @Override
  public void update(Observable o, Object arg) {
    final boolean hasProject = userHasProject();
    final boolean canOpen =
        userHasProject() || userHasOthersProject() || User.getLoggedUser().isAdmin();
    if (arg instanceof SharedProjectEvent) {
      SharedProjectEvent event = (SharedProjectEvent) arg;
      switch (event.getSharingType()) {
          /*
           * Projeto do usuário foi marcado como publico
           */
        case ALL_RO:
        case ALL_RW:
          {
            SwingThreadDispatcher.invokeLater(
                new Runnable() {
                  @Override
                  public void run() {
                    openUserProjMenuItem.setEnabled(true);
                    removeProjMenuItem.setEnabled(true);
                    openProjButton.setEnabled(true);
                  }
                });
            break;
          }
          /*
           * Projeto foi alterado para privado ou foi concedido acesso seletivo
           */
        case PRIVATE:
        case PARTIAL:
          {
            SwingThreadDispatcher.invokeLater(
                new Runnable() {
                  @Override
                  public void run() {
                    openUserProjMenuItem.setEnabled(canOpen);
                    removeProjMenuItem.setEnabled(hasProject);
                    openProjButton.setEnabled(canOpen);
                  }
                });
            break;
          }
      }
    } else if (arg instanceof ProjectUserEvent) {
      final ProjectUserEvent event = (ProjectUserEvent) arg;
      switch (event.type) {
          /*
           * Projeto do usuário foi removido ou Projeto do qual o usuário
           * participava foi removido
           */
        case ProjectUserEvent.DELETE:
          {
            SwingThreadDispatcher.invokeLater(
                new Runnable() {
                  @Override
                  public void run() {
                    openUserProjMenuItem.setEnabled(canOpen);
                    removeProjMenuItem.setEnabled(hasProject);
                    openProjButton.setEnabled(canOpen);

                    CommonProjectInfo info = (CommonProjectInfo) event.item;

                    // Limpa a árvore de projetos se o projeto removido está aberto
                    if ((projectTree.getProject() != null)
                        && (info.projectId.equals(projectTree.getProject().getId()))) {
                      projectTree.resetProject();
                    }
                  }
                });
            break;
          }

          /* Projeto do usuário foi criado */
        case ProjectUserEvent.CREATE:
          {
            SwingThreadDispatcher.invokeLater(
                new Runnable() {
                  @Override
                  public void run() {
                    openUserProjMenuItem.setEnabled(true);
                    removeProjMenuItem.setEnabled(true);
                    openProjButton.setEnabled(true);
                  }
                });
            break;
          }
      }
    }
  }

  /** Finaliza o cliente. */
  public void exitApplication() {
    if (!ClientRemoteMonitor.getInstance().ping()) {
      /*
       * Caso não haja comunicação com o servidor, o usuário é questionado se
       * deseja continuar com o processo de finalização da aplicação.
       */
      int answer =
          tecgraf.javautils.gui.StandardDialogs.showYesNoDialog(
              getDesktopFrame(), getTitle(), LNG.get("desktop.confirm_shutdown"));
      if (answer != JOptionPane.YES_OPTION) {
        return;
      }
    }
    notifPanel.setUser(null);
    shutdownDesktop(new ProjectCloseAction(projectTree));
  }

  /** Instala uma árvore de projetos no desktop principal do cliente. */
  protected void installProjectTree() throws ClientException {
    new BasicProjectTreeFactory();
    projectTree = ProjectTreeFactory.getInstance().makeProjectTree(getDesktopFrame(), getTitle());
    projectTree.addMouseListener();
    /* Listener para reação à mudança de projeto e seleção na árvore */
    projectTree.addProjectTreeListener(
        new ProjectTreeAdapter() {
          /* Mudança de projeto: atualiza caminho selecionado */
          @Override
          public void projectChanged(CommonClientProject proj) {
            applicationPanel.setProjectFlag(proj != null);
          }
        });
  }

  /*--------------------------------------------------------------------------*
   * Métodos para criação dos ítens do menu Projeto                           *
   *--------------------------------------------------------------------------*/
  /**
   * Constrói o submenu para abertura de projetos
   *
   * @return o menu de abertura de projetos.
   */
  protected JMenuItem createOpenMenuItem(boolean hasProject) {
    openUserProjMenuItem = new JMenuItem(new CommonProjectOpenAction(projectTree));
    openUserProjMenuItem.setIcon(ApplicationImages.ICON_OPEN_16);
    openUserProjMenuItem.setText(LNG.get("desktop.menu.project.open"));
    openUserProjMenuItem.setEnabled(hasProject);
    String accelerator = LNG.get("desktop.menu.project.open.accelerator");
    if (null != accelerator) {
      openUserProjMenuItem.setAccelerator(KeyStroke.getKeyStroke(accelerator));
    }
    return openUserProjMenuItem;
  }

  /**
   * Constrói o item de menu para criação de um novo projeto.
   *
   * @return o ítem de menu de criação de projeto
   */
  protected JMenuItem createProjectCreationItem() {
    JMenuItem createMenuItem = new JMenuItem(new CommonProjectCreationAction(projectTree));
    createMenuItem.setIcon(ApplicationImages.ICON_NEW_16);
    createMenuItem.setText(LNG.get("desktop.project.create"));
    createMenuItem.setEnabled(true);
    return createMenuItem;
  }

  /**
   * Constrói o item de menu para remover um projeto.
   *
   * @return o ítem de menu de remoção de projeto
   */
  protected JMenuItem createProjectRemoveItem(boolean hasProject) {
    removeProjMenuItem = new JMenuItem(new ProjectDeleteAction(projectTree));
    removeProjMenuItem.setText(LNG.get("desktop.project.remove"));
    removeProjMenuItem.setEnabled(hasProject);
    return removeProjMenuItem;
  }

  /**
   * Constrói o item de menu para atualização da árvore de projetos.
   *
   * @return o ítem de menu de atualização da árvore do projeto aberto
   */
  protected JMenuItem createUpdateTreeItem() {
    JMenuItem updateTreeMenuItem = new JMenuItem(new ProjectTreeUpdateAction(projectTree));
    updateTreeMenuItem.setEnabled(false);
    enableIfProjectOpened(updateTreeMenuItem);
    return updateTreeMenuItem;
  }

  /**
   * Constrói o item de menu para ver o histórico do projeto.
   *
   * @return o ítem de menu de consulta ao histórico do projeto aberto
   */
  protected JMenuItem createProjectHistoryItem() {
    JMenuItem historyViewMenuItem = new JMenuItem(new ProjectHistoryShowAction(projectTree));
    historyViewMenuItem.setText(LNG.get("desktop.project.history"));
    historyViewMenuItem.setEnabled(false);
    enableIfProjectOpened(historyViewMenuItem);
    return historyViewMenuItem;
  }

  /**
   * Constrói o item de menu para ver informações sobre o projeto.
   *
   * @return o ítem de menu que exibe informações do projeto aberto
   */
  protected JMenuItem createProjectInfoItem() {
    final JMenuItem infoItem = new JMenuItem(new ProjectInfoAction(this));
    infoItem.setText(LNG.get("desktop.menu.project.info"));
    String usageAccelerator = LNG.get("desktop.menu.project.info.accelerator");
    if (null != usageAccelerator) {
      infoItem.setAccelerator(KeyStroke.getKeyStroke(usageAccelerator));
    }
    infoItem.setEnabled(false);
    enableIfProjectOpened(infoItem);
    return infoItem;
  }

  /**
   * Constrói o item de menu para atualização de dados do projeto.
   *
   * @return o ítem de menu de atualização de dados do projeto aberto
   */
  protected JMenuItem createProjectUpdateItem() {
    JMenuItem updateMenuItem = new JMenuItem(new CommonProjectUpdateAction(projectTree));
    updateMenuItem.setText(LNG.get("desktop.project.modify"));
    updateMenuItem.setEnabled(false);
    enableIfProjectOpened(updateMenuItem);
    return updateMenuItem;
  }

  /**
   * Constrói o item de menu para gerenciamento dos usuários de projetos.
   *
   * @return o ítem de menu de controle de acesso de usuários no projeto
   */
  protected JMenuItem createUsersManagementMenuItem() {
    JMenuItem usersManagementMenuItem =
        new JMenuItem(new ProjectUsersManagementAction(projectTree));
    usersManagementMenuItem.setText(LNG.get("desktop.project.access.control"));
    usersManagementMenuItem.setEnabled(false);
    enableIfAdminProject(usersManagementMenuItem);
    return usersManagementMenuItem;
  }

  /**
   * Constrói o item de menu que habilita ou desabilita a exibição de arquivos ocultos (que começam
   * com "."). O estado do checkbox é obtido das preferências do usuário.
   */
  protected JMenuItem createShowHiddenFilesMenuItem() {
    final JCheckBoxMenuItem showHiddenFilesMenuItem =
        new JCheckBoxMenuItem(new ShowHiddenFilesAction(projectTree));

    PreferenceCategory dp = getDesktopPreferences();
    boolean showHiddenFilesPref = dp.getPreferenceAsBoolean(DesktopPref.SHOW_HIDDEN_FILES);
    showHiddenFilesMenuItem.setSelected(showHiddenFilesPref);

    return showHiddenFilesMenuItem;
  }

  /**
   * Constrói o item de menu para fechar o projeto.
   *
   * @return o item de menu para fechar o projeto aberto
   */
  protected JMenuItem createProjectCloseMenuItem() {
    JMenuItem closeMenuItem = new JMenuItem(new ProjectCloseAction(projectTree));
    closeMenuItem.setEnabled(false);
    enableIfProjectOpened(closeMenuItem);
    return closeMenuItem;
  }

  /**
   * Constrói o item de menu para sair do sistema.
   *
   * @return o item de menu para sair do sistema
   */
  protected JMenuItem createExitMenuItem() {
    JMenuItem exitMenuItem = new JMenuItem(LNG.get("desktop.exit"));
    exitMenuItem.addActionListener(
        new ActionListener() {
          @Override
          public void actionPerformed(ActionEvent e) {
            exitApplication();
          }
        });
    return exitMenuItem;
  }

  /*--------------------------------------------------------------------------*
   * Métodos para criação dos ítens do menu Algoritmos                        *
   *--------------------------------------------------------------------------*/
  /**
   * Constrói o item de menu para execução de algoritmos.
   *
   * @return o item de menu para executar um algoritmo remotamente.
   */
  protected JMenuItem createExecuteMenuItem() {
    JMenuItem execMenuItem = new JMenuItem(LNG.get("desktop.algorithms.execute"));
    execMenuItem.addActionListener(applicationPanel.getApplicationAction("executor"));
    execMenuItem.setEnabled(false);
    execMenuItem.setIcon(ApplicationImages.ICON_EXECUTE_CMD_16);
    enableIfProjectOpened(execMenuItem);
    return execMenuItem;
  }

  /**
   * Constrói o item de menu para monitoração da execução de algoritmos.
   *
   * @return o item de menu para monitorar a execução de algoritmos.
   */
  protected JMenuItem createComandMonitoringMenuItem() {
    JMenuItem cmdsMenuItem =
        new JMenuItem(applicationPanel.getApplicationAction("commandsmonitor"));
    cmdsMenuItem.setText(LNG.get("desktop.menu.algoritm.monitor"));
    return cmdsMenuItem;
  }

  /**
   * Constrói o item de menu para a aplicação de construtor de fluxos.
   *
   * @return o item de menu para construir fluxo de algoritmos.
   */
  protected JMenuItem createAlgoFlowMenuItem() {
    JMenuItem cmdsMenuItem =
        new JMenuItem(applicationPanel.getApplicationAction("flowapplication"));
    cmdsMenuItem.setText(LNG.get("desktop.menu.algorithm.flow"));
    return cmdsMenuItem;
  }

  /**
   * Constrói o item de menu para gerenciamento do repositório de algoritmos. Essa opção está
   * disponível apenas para os administradores ou para usuários com permissão de administração do
   * repositório de algoritmos.
   *
   * @return o item de menu para gerenciamento dos algoritmos no repositório
   */
  protected JMenuItem createAlgorithmsAdministrationMenuItem() {
    JMenuItem algorithmMenuItem =
        new JMenuItem(applicationPanel.getApplicationAction(ALGORITHMS_MANAGER_APPLICATION_ID));
    algorithmMenuItem.setText(LNG.get("desktop.admin.algorithms"));
    return algorithmMenuItem;
  }

  /**
   * Constrói o item de menu para gerenciamento dos arquivos de log. Essa opção está disponível
   * apenas para os administradores ou para usuários com permissão de administração do repositório
   * de algoritmos.
   *
   * @return o item de menu para gerenciamento dos arquivos de log
   */
  protected JMenuItem createLogAdministrationMenuItem() {
    JMenuItem logMenuItem =
        new JMenuItem(applicationPanel.getApplicationAction("logadministration"));
    logMenuItem.setText(LNG.get("desktop.admin.logs"));
    return logMenuItem;
  }

  /**
   * Cria um item de menu para a aplicação Gerenciador de Algoritmos.
   *
   * @return o item de menu criado
   */
  private JMenuItem createApplicationReloadMenuItem() {
    final AbstractAction action =
        new AbstractAction() {
          @Override
          public void actionPerformed(final ActionEvent event) {
            ApplicationManagementProxy.reloadApplications(getDesktopFrame());
          }
        };
    final String text = LNG.get("desktop.menu.admin.applications.reload");
    final JMenuItem menuItem = new JMenuItem(action);
    menuItem.setText(text);
    return menuItem;
  }

  /*--------------------------------------------------------------------------*
   * Métodos para criação dos ítens do menu Servidores (SGAs)                 *
   *--------------------------------------------------------------------------*/
  /**
   * Constrói o item de menu para gerenciamento e monitoração dos servidores SGAs.
   *
   * @return o item de menu para gerenciar e monitorar os servidores.
   */
  protected JMenuItem createServersMenuItem() {
    JMenuItem serversMenuItem = new JMenuItem(applicationPanel.getApplicationAction("sgamonitor"));
    serversMenuItem.setText(LNG.get("desktop.monitoring.servers"));
    return serversMenuItem;
  }

  /*--------------------------------------------------------------------------*
   * Métodos para criação dos ítens do menu de Configuração                   *
   *--------------------------------------------------------------------------*/

  /**
   * Constrói o item de menu para alterar a configuração do usuário corrente. No caso do usuário ser
   * administrador, o diálogo permite que ele altere permissões além da própria senha.
   *
   * @return o item de menu para configurar o usuário corrente.
   */
  protected JMenuItem createUserConfigItem() {
    JMenuItem configUserMenuItem = new JMenuItem(LNG.get("desktop.menu.config.user"));
    configUserMenuItem.addActionListener(
        new ActionListener() {
          @Override
          public void actionPerformed(ActionEvent e) {
            JFrame frame = getInstance().getDesktopFrame();
            User loggedUser = User.getLoggedUser();
            UserInfoDialog dialog;
            if (loggedUser.isAdmin()) {
              dialog = new AdminModifyUserInfoDialog(frame, loggedUser.getId());
            } else {
              dialog = new ModifyUserInfoDialog(frame, loggedUser.getId());
            }
            dialog.setVisible(true);
          }
        });
    return configUserMenuItem;
  }

  /**
   * Constrói o menu com as opções de configuração das janelas com as aplicações do desktop.
   *
   * @return o menu para configurar a exibição das aplicações no desktop.
   */
  protected JMenu createGroupsConfigurationMenu() {
    return applicationPanel.createGroupsConfigurationMenu();
  }

  /**
   * Constrói o item de menu de administração do sistema.
   *
   * @return o item de menu para administrar o sistema.
   */
  protected JMenuItem createAdminMenuItem() {
    JMenuItem adminMenuItem =
        new JMenuItem(
            MessageFormat.format(
                LNG.get("desktop.menu.system.admin"), Client.getInstance().getSystemName()));
    adminMenuItem.addActionListener(
        new ActionListener() {
          @Override
          public void actionPerformed(ActionEvent e) {
            if (adminFrame == null) {
              adminFrame = new AdminFrame();
            }
            adminFrame.showUserManagement();
          }
        });
    return adminMenuItem;
  }

  /*--------------------------------------------------------------------------*
   * Métodos para criação dos ítens do menu de Ajuda                          *
   *--------------------------------------------------------------------------*/
  /**
   * Constrói o item de menu para exibir o diálogo de "About".
   *
   * @return o item de menu para exibir o diálogo com as informações sobre o sistema
   */
  protected JMenuItem createAboutMenuItem() {
    JMenuItem aboutMenuItem = new JMenuItem(new AboutAction());
    aboutMenuItem.setText(
        MessageFormat.format(LNG.get("desktop.menu.about"), Client.getInstance().getSystemName()));
    return aboutMenuItem;
  }
}
